/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

class ForecastController {

    static allowedMethods = [save: "POST", update: "POST", delete: "POST"]

    def index = {
        redirect(action: "list", params: params)
    }

    def list = {
        params.max = Math.min(params.max ? params.int('max') : 10, 100)
        [forecastInstanceList: Forecast.list(params), forecastInstanceTotal: Forecast.count()]
    }

    def create = {
        def forecastInstance = new Forecast()
        forecastInstance.properties = params
        return [forecastInstance: forecastInstance]
    }

    def save = {
        def forecastInstance = new Forecast(params)
        if (forecastInstance.save(flush: true)) {
            flash.message = "${message(code: 'default.created.message', args: [message(code: 'forecast.label', default: 'Forecast'), forecastInstance.id])}"
            redirect(action: "show", id: forecastInstance.id)
        }
        else {
            render(view: "create", model: [forecastInstance: forecastInstance])
        }
    }

    def show = {
        def forecastInstance = Forecast.get(params.id)
        if (!forecastInstance) {
            flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'forecast.label', default: 'Forecast'), params.id])}"
            redirect(action: "list")
        }
        else {
            [forecastInstance: forecastInstance]
        }
    }

    def edit = {
        def forecastInstance = Forecast.get(params.id)
        if (!forecastInstance) {
            flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'forecast.label', default: 'Forecast'), params.id])}"
            redirect(action: "list")
        }
        else {
            return [forecastInstance: forecastInstance]
        }
    }

    def update = {
        def forecastInstance = Forecast.get(params.id)
        if (forecastInstance) {
            if (params.version) {
                def version = params.version.toLong()
                if (forecastInstance.version > version) {
                    
                    forecastInstance.errors.rejectValue("version", "default.optimistic.locking.failure", [message(code: 'forecast.label', default: 'Forecast')] as Object[], "Another user has updated this Forecast while you were editing")
                    render(view: "edit", model: [forecastInstance: forecastInstance])
                    return
                }
            }
            forecastInstance.properties = params
            if (!forecastInstance.hasErrors() && forecastInstance.save(flush: true)) {
                flash.message = "${message(code: 'default.updated.message', args: [message(code: 'forecast.label', default: 'Forecast'), forecastInstance.id])}"
                redirect(action: "show", id: forecastInstance.id)
            }
            else {
                render(view: "edit", model: [forecastInstance: forecastInstance])
            }
        }
        else {
            flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'forecast.label', default: 'Forecast'), params.id])}"
            redirect(action: "list")
        }
    }

    def delete = {
        def forecastInstance = Forecast.get(params.id)
        if (forecastInstance) {
            try {
                forecastInstance.delete(flush: true)
                flash.message = "${message(code: 'default.deleted.message', args: [message(code: 'forecast.label', default: 'Forecast'), params.id])}"
                redirect(action: "list")
            }
            catch (org.springframework.dao.DataIntegrityViolationException e) {
                flash.message = "${message(code: 'default.not.deleted.message', args: [message(code: 'forecast.label', default: 'Forecast'), params.id])}"
                redirect(action: "show", id: params.id)
            }
        }
        else {
            flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'forecast.label', default: 'Forecast'), params.id])}"
            redirect(action: "list")
        }
    }
}
