/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

class OrderbookController {

  static allowedMethods = [save: "POST", update: "POST", delete: "POST"]

  def index = {
    redirect(action: "list", params: params)
  }

  def list = {
    params.max = Math.min(params.max ? params.int('max') : 10, 100)
    flash.history = params.history
    if (params.product) try {flash.product = params.product as Long} catch (Exception e) {}

    def query
    def results
    def criteria = Product.createCriteria()

    query = {
        if (flash.product) {
          eq('id', flash.product)
        }
    }
    results = criteria.list(params, query)

    [productInstanceList: results, history: flash.history]
  }

  def show = {
    def orderbookInstance = Orderbook.get(params.id)
    if (!orderbookInstance) {
      flash.error = "${message(code: 'default.not.found.message', args: [message(code: 'orderbook.label', default: 'Orderbook'), params.id])}"
      redirect(action: "list")
    }
    else {
      [orderbookInstance: orderbookInstance]
    }
  }
  
  def latest = {
    Boolean errorOccured = true

    def orderbookProduct = Product.get(params.id)
    def orderbookInstance
    if(orderbookProduct){
      orderbookInstance = Orderbook.withCriteria(uniqueResult: true){
        maxResults(1)
        eq('product', orderbookProduct)
        eq('outdated', false)
      }
      if (orderbookInstance){
        errorOccured = false
      }
    }

    if (errorOccured){
      flash.error = "No orderbook found. Most likely for this product not trade has occurred yet. Please check back later."
      redirect(action: "list")
    }
    else {
      render(view:"show", model: [orderbookInstance: orderbookInstance])
    }
  }

}
