/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

class ShoutController {

  def auctionService

  static allowedMethods = [save: "POST", update: "POST", delete: "POST"]

  def index = {
    redirect(action: "list", params: params)
  }

  def list = {
    params.max = Math.min(params.max ? params.int('max') : 10, 100)


    flash.outdated = params.outdated
    if (params.product) try {flash.product = params.product as Long} catch (Exception e) {}
    if (params.person) try {flash.person = params.person as Long} catch (Exception e) {}
    if (params.modReasonCode) try {flash.modReasonCode = params.modReasonCode as ModReasonCode} catch (Exception e) {}

    def query
    def results
    def criteria = Shout.createCriteria()

    query = {
      and {
        if (!params.outdated) eq('outdated', false)
        if (flash.product) {
          def selectedProduct = Product.get(flash.product)
          eq('product', selectedProduct)
        }
        if (flash.person) {
          def selectedPerson = Person.get(flash.person)
          eq('person', selectedPerson)
        }
        if (flash.modReasonCode) {
          eq('modReasonCode', flash.modReasonCode)
        }
      }
    }
    results = criteria.list(params, query)

    [shoutInstanceList: results]
  }

  def create = {
    def shoutInstance = new Shout()
    shoutInstance.properties = params
    return [shoutInstance: shoutInstance]
  }

  def save = {

    def shoutInstance = new Shout(params)

    //check for marketorder && limitPrice
    if (shoutInstance?.orderType == OrderType.MARKET && shoutInstance?.limitPrice) {
      flash.message = "Error: you cannot set a limit price to a market order!"
      render(view: "create", model: [shoutInstance: shoutInstance])
    } else {

      //check for scale longer than 2 digits in limitPrice
      if (shoutInstance?.limitPrice?.scale > 2) {
        flash.message = "Error: the scale of limitPrice has to be 2 digits"
        render(view: "create", model: [shoutInstance: shoutInstance])
      } else {

        //call auctionService
        try {
          auctionService.addShout(shoutInstance)
        } catch (Exception e) {
          flash.error = "An error occurred while processing the order: No order was added"
          render(view: "create", model: [shoutInstance: shoutInstance])
        }
        flash.message = "${message(code: 'default.created.message', args: [message(code: 'shout.label', default: 'Shout'), shoutInstance.id])}"
        redirect(action: "create")
      }
    }
  }

  def show = {
    def shoutInstance = Shout.get(params.id)
    if (!shoutInstance) {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
      redirect(action: "list")
    }
    else {
      [shoutInstance: shoutInstance]
    }
  }

  def edit = {
    def shoutInstance = Shout.get(params.id)
    if (!shoutInstance) {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
      redirect(action: "list")
    }
    else {
      return [shoutInstance: shoutInstance]
    }
  }


  def update = {

    def shoutInstance = Shout.get(params.id)
    if (shoutInstance) {
      if (params.version) {
        def version = params.version.toLong()
        if (shoutInstance.version > version) {

          shoutInstance.errors.rejectValue("version", "default.optimistic.locking.failure", [message(code: 'shout.label', default: 'Shout')] as Object[], "Another user has updated this Shout while you were editing")
          render(view: "edit", model: [shoutInstance: shoutInstance])
          return
        }
      }
      shoutInstance.properties = params
      shoutInstance.discard()  //prevent grails from saving

      try {
        def modShout = auctionService.editShout(shoutInstance)
        flash.message = "${message(code: 'default.updated.message', args: [message(code: 'shout.label', default: 'Shout'), shoutInstance.id])}"
        redirect(action: "show", id: modShout.id) //show modified shout
      } catch (Exception e) {
        flash.error "An error occurred while processing the order: ${e.cause}"
        render(view: "edit", model: [shoutInstance: shoutInstance])
      }
    } else {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
      redirect(action: "list")
    }
  }

  def delete = {
    def shoutInstance = Shout.get(params.id)
    if (shoutInstance) {
      try {
        auctionService.deleteShout(shoutInstance)
        flash.message = "${message(code: 'default.deleted.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
        redirect(action: "list")
      }
      catch (org.springframework.dao.DataIntegrityViolationException e) {
        flash.message = "${message(code: 'default.not.deleted.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
        redirect(action: "show", id: params.id)
      }
    }
    else {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'shout.label', default: 'Shout'), params.id])}"
      redirect(action: "list")
    }
  }
}
