/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

class TransactionLogController {

  static allowedMethods = [save: "POST", update: "POST", delete: "POST"]

  def index = {
    redirect(action: "list", params: params)
  }

  def trades = {
    params.type='TRADE'
    redirect(action: 'list', params: params)
  }

  def quotes = {
    params.type='QUOTE'
    redirect(action: 'list', params: params)
  }
  
  def list = {
    params.max = Math.min(params.max ? params.int('max') : 10, 100)

    if (params.product) try {flash.product = params.product as Long} catch (Exception e) {}
    if (params.type) try {flash.type = params.type as TransactionType} catch (Exception e) {}


    def query
    def results
    def criteria = TransactionLog.createCriteria()
    
    query = {
      and {
        if (flash.product) {
          def selectedProduct = Product.get(flash.product)
          eq('product', selectedProduct)
        }
        if (flash.type) {
          eq('type', flash.type)
        }
      }
    }
    results = criteria.list(params, query)
    [transactionLogInstanceList: results]
  }

  def show = {
    def transactionLogInstance = TransactionLog.get(params.id)
    if (!transactionLogInstance) {
      flash.message = "${message(code: 'default.not.found.message', args: [message(code: 'transactionLog.label', default: 'TransactionLog'), params.id])}"
      redirect(action: "list")
    }
    else {
      [transactionLogInstance: transactionLogInstance]
    }
  }
}
