/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.CashPositionNotification
import grails.converters.*

class CashPosition {

  def idGeneratorService

  Competition competition
  Person person
  BigDecimal change
  BigDecimal balance
  String description
  Date dateCreated = new Date()
  Long transactionID
  Boolean latest = true

  static belongsTo = [person: Person]

  static constraints = {
    competition(nullable: true)
    person(nullable: false)
    change(nullable: false, scale: 2)
    balance(nullable: false, scale: 2)
    description(nullable: true)
    dateCreated(nullable: false)
    transactionID(nullable: false)
    latest(nullable: false)
  }


  static mapping = {
    change column: "relative_change"
  }

  static transients = ['idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }

  public String toString() {
    return "${person}-${change}-${description}-${dateCreated}"
  }

  public CashPositionNotification toCashPositionNotification() {
    def cashPositionNotification = new CashPositionNotification()
    cashPositionNotification.serverId = this.id
    cashPositionNotification.competitionId = this.competition?.id
    cashPositionNotification.competitionName = this.competition?.name
    cashPositionNotification.change = this.change
    cashPositionNotification.balance = this.balance
    cashPositionNotification.description = this.description
    cashPositionNotification.dateCreated = this.dateCreated.getTime()
    cashPositionNotification.transactionID = this.transactionID
    cashPositionNotification.latest = this.latest
    return cashPositionNotification
  }

  public String toCashPositionNotificationXml() {
     return toCashPositionNotification() as XML
  }
}
