/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.CompetitionNotification
import grails.converters.*

class Competition {

  String name
  Boolean enabled = false
  Boolean current = false
  Date currentCompetitionTime
  CompetitionStatus competitionStatus = CompetitionStatus.Created
  String description
  Long initialDuration = 60l //seconds before and initial deactivation / activation cylce is started
  Long durationBetweenShifts = 60l //seconds between deactivation of oldest and activation of newest product
  Integer timeslotLength = 60l //minutes
  Integer timeslotsOverall = 24l //overall timeslotsOverall
  Integer timeslotsOpen = 12l //concurrently open timeslotsOverall (time window)
  Integer deactivateTimeslotsAhead = 1
  Date simulationStartTime = new Date()
  Integer participantCount = 6
  Date dateCreated = new Date()
  Date lastUpdated = new Date()
  BigDecimal balancingCostOver = 0
  BigDecimal balancingCostUnder = 0
  String queueNames

  static constraints = {
    name(unique: true, blank: false)
    enabled(nullable: false)
    current(nullable: true)
    currentCompetitionTime (nullable: true)
    competitionStatus(nullable: false)
    description(nullable: true)
    initialDuration(nullable: false, min: 1l)
    durationBetweenShifts(nullable: false, min: 1l)
    timeslotLength(nullable: false, min: 1)
    timeslotsOverall(nullable: false, min: 1)
    timeslotsOpen(nullable: false, min: 1, validator: { timeslotsOpen, competition ->
      timeslotsOpen <= (competition.timeslotsOverall - competition.deactivateTimeslotsAhead)
    })
    deactivateTimeslotsAhead(nullable: false, min: 0, validator: {deactivateTimeslotsAhead, competition ->
      deactivateTimeslotsAhead <= (competition.timeslotsOverall - competition.timeslotsOpen)
    })
    simulationStartTime(nullable: false)
    participantCount(nullable: false, min: 0, max: 20)
    dateCreated(nullable: false)
    lastUpdated(nullable: false)
    balancingCostOver(nullable: false, scale: 2)
    balancingCostUnder(nullable: false, scale: 2)
    queueNames(nullable: true)
  }

  public String toString() {
    return name
  }

  public CompetitionNotification toCompetitionNotification() {
    def competitionNotification = new CompetitionNotification()
    competitionNotification.serverId = this.id
    competitionNotification.name = this.name
    competitionNotification.competitionStatus = this.competitionStatus
    competitionNotification.description = this.description
    competitionNotification.current = this.current
    competitionNotification.currentCompetitionTime = this.currentCompetitionTime?.getTime()
    competitionNotification.initialDuration = this.initialDuration
    competitionNotification.durationBetweenShifts = this.durationBetweenShifts
    competitionNotification.timeslotLength = this.timeslotLength
    competitionNotification.timeslotsOverall = this.timeslotsOverall
    competitionNotification.timeslotsOpen = this.timeslotsOpen
    competitionNotification.deactivateTimeslotsAhead = this.deactivateTimeslotsAhead
    competitionNotification.simulationStartTime = this.simulationStartTime?.getTime()
    competitionNotification.participantCount = this.participantCount
    competitionNotification.balancingCostOver = this.balancingCostOver
    competitionNotification.balancingCostUnder = this.balancingCostUnder
    return competitionNotification
  }

  public String toCompetitionNotificationXml() {
    return toCompetitionNotification() as XML
  }
}
