/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.DepotPositionNotification
import grails.converters.*

class DepotPosition {

  def idGeneratorService

  Competition competition
  Person person
  Integer change
  Integer balance
  String description
  Date dateCreated = new Date();
  Product product
  Long transactionID
  Boolean latest = true

  static belongsTo = [person: Person, product: Product]

  static constraints = {
    competition(nullable: true)
    person(nullable: false)
    change(nullable: false)
    balance(nullable: false)
    description(nullable: true)
    dateCreated(nullable: false)
    product(nullable: false)
    transactionID(nullable: false)
    latest(nullable: false)
  }

  static mapping = {
    change column: "relative_change"
  }

  static transients = ['idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }

  public String toString() {
    return "${person}-${product}-${change}-${description}-${dateCreated}"
  }

  public DepotPositionNotification toDepotPositionNotification() {
    def depotPositionNotification = new DepotPositionNotification()
    depotPositionNotification.serverId = this.id
    depotPositionNotification.competitionId = this.competition?.id
    depotPositionNotification.competitionName = this.competition?.name
    depotPositionNotification.productId = this.product?.id
    depotPositionNotification.productName = this.product?.name
    depotPositionNotification.productStartDateTime = this.product?.startDateTime?.getTime()
    depotPositionNotification.productEndDateTime = this.product?.endDateTime?.getTime()
    depotPositionNotification.change = this.change
    depotPositionNotification.balance = this.balance
    depotPositionNotification.description = this.description
    depotPositionNotification.dateCreated = this.dateCreated.getTime()
    depotPositionNotification.transactionID = this.transactionID
    depotPositionNotification.latest = this.latest
    return depotPositionNotification
  }

  public String toDepotPositionNotificationXml() {
    toDepotPositionNotification() as XML
  }

}
