/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.ForecastNotification
import grails.converters.*

class Forecast {

  def idGeneratorService

  Competition competition
  Person person
  Product product
  Date competitionTime
  BigDecimal forecastValue
  BigDecimal trueValue

  static belongsTo = [person: Person, product: Product]

  static constraints = {
    competition(nullable: true)
    person(nullable: false)
    product(nullable: false)
    competitionTime(nullable: false)
    forecastValue(nullable: false, scale: 2)
    trueValue(nullable: false, scale: 2)
  }

  static mapping = {
    person(index: 'forecast_person_product_competition_time_idx')
    product(index: 'forecast_person_product_competition_time_idx')
    competitionTime(index: 'forecast_person_product_competition_time_idx')
  }

  static transients = ['idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }

  public ForecastNotification toForecastNotification() {
    def forecastNotification = new ForecastNotification()
    forecastNotification.serverId = this.id
    forecastNotification.competitionId = this.competition?.id
    forecastNotification.competitionName = this.competition?.name
    forecastNotification.productId = this.product?.id
    forecastNotification.productName = this.product?.name
    forecastNotification.productStartDateTime = this.product?.startDateTime?.getTime()
    forecastNotification.productEndDateTime = this.product?.endDateTime?.getTime()
    forecastNotification.competitionTime = this.competitionTime?.getTime()
    forecastNotification.forecastValue = this.forecastValue
    return forecastNotification
  }

  public String toForecastNotificationXml() {
    return toForecastNotification() as XML
  }

}
