/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import org.apache.commons.lang.RandomStringUtils

class Person {

  def idGeneratorService

  Competition competition
  String username //user name
  Boolean enabled = false
  Boolean ready = false
	String userRealName //user real name
	String passwd //md5 apiKey
  String email
	String description = '' //short (optional) description
	String pass = '[secret]' //plain apiKey to create a MD5 apiKey -> not stored in db
  String apiKey = RandomStringUtils.randomAlphanumeric(10).toString() //used for API authentication instead of apiKey
  Double aMin = 0.05
  Double aMax = 0.15
  Double alpha = 0.5
  Long forecastSeed = new GregorianCalendar().getTimeInMillis()

  static hasMany = [cashPositions: CashPosition, depotPositions: DepotPosition, shout: Shout, authorities: Role]
  static belongsTo = Role

  static constraints = {
    competition (nullable: true)
    username(nullable: false, blank: false, unique:true)
    userRealName (nullable: false, blank: false)
    passwd (nullable: false, blank: false)
    email(nullable: true, email: true)
    enabled(nullable: false)
    ready(nullable: false)
    apiKey(blank: false, unique: false, minSize: 6)
    forecastSeed(nullable: false)
  }

  static transients = ['pass','idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }


  public void setEnabled(Boolean enabled) {
    if (!enabled && this.id != null) {
      def shouts = Shout.findAllByPersonAndOutdated(this, false)
      shouts?.each {
        it.setOutdated(true)
        it.setModReasonCode(ModReasonCode.DELETIONBYSYSTEM)
        it.save()
      }
    }
    this.enabled = enabled
  }

  
  public String toQueueName() {
    return "users.${this.username}.outputQueue";
  }


  public String toString() {
    return username;
  }

}
