/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.ProductNotification
import grails.converters.*

class Product {

  def idGeneratorService

  Competition competition
  String name
  Boolean enabled = false
  Date startDateTime = new Date()
  Date endDateTime = new Date()
  Long serialNumber
  Boolean current = false
  SortedSet orderbooks

  static hasMany = [depotPositions: DepotPosition, forecasts: Forecast, shouts: Shout, orderbooks: Orderbook]

  static constraints = {
    competition(nullable: true)
    name(nullable: false, blank: false)
    startDateTime(nullable: false)
    endDateTime(nullable: false)
    current(nullable: true)
    serialNumber(nullable: false)
  }

  static transients = ['idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }

  //When enabled is set 'false' all connected shouts are set to outdated = 'true' and to modReasonCode DELETIONBYSYSTEM

  def afterUpdate() {
    if (!enabled) {
      def results = Shout.findAllByProductAndOutdated(this, false)
      results.each {
        it.setOutdated(true)
        it.setModReasonCode(ModReasonCode.DELETIONBYSYSTEM)
      }
    }
  }

  public String toString() {
    return name;
  }

  public Product next() {
    if (this.competition) {
      return Product.findByCompetitionAndSerialNumber(this.competition, this.serialNumber + 1l)
    } else {
      return Product.findByCompetitionIsNullAndSerialNumber(this.serialNumber + 1l)
    }
  }

  public Product previous() {
    if (this.competition) {
      return Product.findByCompetitionAndSerialNumber(this.competition, this.serialNumber - 1l)
    } else {
      return Product.findByCompetitionIsNullAndSerialNumber(this.serialNumber - 1l)
    }
  }

  public ProductNotification toProductNotification() {
    def productNotification = new ProductNotification()
    productNotification.serverId = this.id
    productNotification.competitionId = this.competition?.id
    productNotification.competitionName = this.competition?.name
    productNotification.name = this.name
    productNotification.enabled = this.enabled
    productNotification.startDateTime = this.startDateTime.getTime()
    productNotification.endDateTime = this.endDateTime.getTime()
    productNotification.serialNumber = this.serialNumber
    productNotification.current = this.current
    return productNotification
  }

  public String toProductNotificationXml() {
    return toProductNotification() as XML
  }
}
