/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.ShoutNotification
import grails.converters.*

class Shout {

  def idGeneratorService

  Competition competition
  Person person
  Product product
  BuySellIndicator buySellIndicator
  BigDecimal remainingQuantity
  BigDecimal limitPrice
  BigDecimal executionQuantity
  BigDecimal executionPrice
  OrderType orderType = OrderType.MARKET
  Date dateCreated = new Date()
  Date dateMod = new Date()
  ModReasonCode modReasonCode = ModReasonCode.INSERT
  Boolean outdated = false
  Long transactionID
  Long shoutID
  AuctionTradeFlag auctionTradeFlag = AuctionTradeFlag.C

  static belongsTo = [person: Person, product: Product]

  static constraints = {
    competition(nullable: true)
    person(nullable: false,
        validator: { person ->
          person.enabled
        })
    product(nullable: false,
        validator: { product ->
          product.enabled
        })
    buySellIndicator(nullable: false)
    remainingQuantity(nullable: false, min: 0.0, scale: 2)
    limitPrice(nullable: true, min: 0.0, scale: 2, validator: { limitPrice, sh1 ->
      (
      (sh1.orderType == OrderType.MARKET && limitPrice == null)
          || (sh1.orderType == OrderType.LIMIT && limitPrice != null)
      )
    })
    executionQuantity(nullable: true, min: 0.0, scale: 2, validator: { executionQuantity, sh1 ->
      (
      ((sh1.modReasonCode == ModReasonCode.EXECUTION || sh1.modReasonCode == ModReasonCode.PARTIALEXECUTION) && executionQuantity != null)
          || ((sh1.modReasonCode != ModReasonCode.EXECUTION && sh1.modReasonCode != ModReasonCode.PARTIALEXECUTION) && executionQuantity == null)
      )
    })
    executionPrice(nullable: true, min: 0.0, scale: 2, validator: { executionPrice, sh1 ->
      (
      ((sh1.modReasonCode == ModReasonCode.EXECUTION || sh1.modReasonCode == ModReasonCode.PARTIALEXECUTION) && executionPrice != null)
          || ((sh1.modReasonCode != ModReasonCode.EXECUTION && sh1.modReasonCode != ModReasonCode.PARTIALEXECUTION) && executionPrice == null)
      )
    })
    orderType(nullable: false)
    dateCreated(nullable: false)
    dateMod(nullable: false)
    modReasonCode(nullable: false)
    outdated(nullable: false)
    transactionID(nullable: false, min: 0L)
    auctionTradeFlag(nullable: true)
    shoutID(nullable: false, min: 0L)
  }

  static transients = ['idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }

  public Shout initModification(ModReasonCode newModReasonCode) {
    def newShout = new Shout()
    newShout.person = this.person
    newShout.product = this.product
    newShout.buySellIndicator = this.buySellIndicator
    newShout.remainingQuantity = this.remainingQuantity
    newShout.limitPrice = this.limitPrice
    newShout.executionQuantity = null
    newShout.executionPrice = null
    newShout.orderType = this.orderType
    newShout.dateCreated = this.dateCreated
    newShout.dateMod = new Date()
    newShout.modReasonCode = newModReasonCode
    newShout.outdated = this.outdated
    newShout.transactionID = null
    newShout.shoutID = this.shoutID
    newShout.auctionTradeFlag = this.auctionTradeFlag

    return newShout
  }

  public void deleteCompleteOrder() {
    def allOrderParts = Shout.findAllByShoutID(this.shoutID)
    for (Shout orderPart: allOrderParts) {
      orderPart.delete()
    }
  }

  public ShoutNotification toShoutNotification() {
    def shoutNotification = new ShoutNotification()
    shoutNotification.serverId = this.id
    shoutNotification.competitionId = this.competition?.id
    shoutNotification.competitionName = this.competition?.name
    shoutNotification.productId = this.product?.id
    shoutNotification.productName = this.product?.name
    shoutNotification.productStartDateTime = this.product?.startDateTime?.getTime()
    shoutNotification.productEndDateTime = this.product?.endDateTime?.getTime()
    shoutNotification.buySellIndicator = this.buySellIndicator
    shoutNotification.remainingQuantity = this.remainingQuantity
    shoutNotification.limitPrice = this.limitPrice
    shoutNotification.executionQuantity = this.executionQuantity
    shoutNotification.executionPrice = this.executionPrice
    shoutNotification.orderType = this.orderType
    shoutNotification.dateCreated = this.dateCreated?.getTime()
    shoutNotification.dateMod = this.dateMod?.getTime()
    shoutNotification.modReasonCode = this.modReasonCode
    shoutNotification.outdated = this.outdated
    shoutNotification.transactionID = this.transactionID
    shoutNotification.shoutID = this.shoutID
    shoutNotification.auctionTradeFlag = this.auctionTradeFlag
    return shoutNotification
  }

  public String toShoutNotificationXml() {
    return toShoutNotification() as XML
  }
}
