/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import edu.kit.iism.cdamarket.notification.QuoteLogNotification
import edu.kit.iism.cdamarket.notification.TradeLogNotification
import grails.converters.XML

class TransactionLog {

  def idGeneratorService

  Competition competition
  Product product
  TransactionType type
  Date dateCreated = new Date()
  Long transactionID

  //Trade properties
  BigDecimal price
  BigDecimal quantity
  Person buyerID
  Person sellerID
  BuySellIndicator buySellIndicator

  //Quote properties
  BigDecimal bid
  BigDecimal bidSize
  BigDecimal ask
  BigDecimal askSize


  static constraints = {
    competition(nullable: true)
    product(nullable: false)
    type(nullable: false)
    dateCreated(nullable: false)
    transactionID(nullable: false)

    price(nullable: true, scale: 2, validator: { price, tl ->
      (tl.type == TransactionType.QUOTE && !price) || (tl.type == TransactionType.TRADE && price)
    })
    quantity(nullable: true, scale: 2, min: 0.0, validator: { quantity, tl ->
      (tl.type == TransactionType.QUOTE && !quantity) || (tl.type == TransactionType.TRADE && quantity)
    })
    buyerID(nullable: true, validator: { buyerID, tl ->
      (tl.type == TransactionType.QUOTE && !buyerID) || (tl.type == TransactionType.TRADE && buyerID)
    })
    sellerID(nullable: true, validator: { sellerID, tl ->
      (tl.type == TransactionType.QUOTE && !sellerID) || (tl.type == TransactionType.TRADE && sellerID)
    })
    buySellIndicator(nullable: true, validator: { bsInd, tl ->
      (tl.type == TransactionType.QUOTE && !bsInd) || (tl.type == TransactionType.TRADE && bsInd)
    })

    bid(nullable: true, scale: 2, validator: { bid, tl ->
      (tl.type == TransactionType.TRADE && !bid) || TransactionType.QUOTE
    })
    bidSize(nullable: true, scale: 2, validator: { bidSize, tl ->
      (tl.type == TransactionType.TRADE && !bidSize) || TransactionType.QUOTE
    })
    ask(nullable: true, scale: 2, validator: { ask, tl ->
      (tl.type == TransactionType.TRADE && !ask) || TransactionType.QUOTE
    })
    askSize(nullable: true, scale: 2, validator: { askSize, tl ->
      (tl.type == TransactionType.TRADE && !askSize) || TransactionType.QUOTE
    })

  }

  static transients = ['idGeneratorService']

  def beforeInsert() {
    this.competition = this.idGeneratorService?.competition
  }


  String toString() {
    return "${dateCreated}-${type}-${product.name}"
  }

  public QuoteLogNotification toQuoteLogNotification() {
    def quoteLogNotification = new QuoteLogNotification()
    quoteLogNotification.serverId = this.id
    quoteLogNotification.competitionId = this.competition?.id
    quoteLogNotification.competitionName = this.competition?.name
    quoteLogNotification.productId = this.product?.id
    quoteLogNotification.productName = this.product?.name
    quoteLogNotification.productStartDateTime = this.product?.startDateTime?.getTime()
    quoteLogNotification.productEndDateTime = this.product?.endDateTime?.getTime()
    quoteLogNotification.bid = this.bid
    quoteLogNotification.bidSize = this.bidSize
    quoteLogNotification.ask = this.ask
    quoteLogNotification.askSize = this.askSize
    return quoteLogNotification
  }

  public String toQuoteLogNotificationXml() {
    return toQuoteLogNotification() as XML
  }

  public TradeLogNotification toTradeLogNotification() {
    def tradeLogNotification = new TradeLogNotification()
    tradeLogNotification.serverId = this.id
    tradeLogNotification.competitionId = this.competition?.id
    tradeLogNotification.competitionName = this.competition?.name
    tradeLogNotification.productId = this.product?.id
    tradeLogNotification.productName = this.product?.name
    tradeLogNotification.productStartDateTime = this.product?.startDateTime?.getTime()
    tradeLogNotification.productEndDateTime = this?.product?.endDateTime?.getTime()
    tradeLogNotification.price = this.price
    tradeLogNotification.quantity = this.quantity
    return tradeLogNotification
  }

  public String toTradeLogNotificationXml() {
    return toTradeLogNotification() as XML
  }
}
