/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

class BalancingPowerService {

    def queueOutputService
  
    boolean transactional = true

    def balance(Product product) {
      //Get max/min trade price
      TransactionLog maxProductPrice = TransactionLog.withCriteria (uniqueResult: true){
        maxResults(1)
        eq('product', product)
        order("price", "desc")
      }

      if (!maxProductPrice){
        //no trades! => no settlement
        return
      }

      //balance each person
      def enabledPersons = Person.withCriteria {
        eq('enabled', true)
      }
      
      for (Person person in enabledPersons) {
         //get forecast
        Forecast currentForecast = Forecast.withCriteria (uniqueResult: true){
          maxResults(1)
          eq('person', person)
          eq('product', product)
        }
        BigDecimal trueValue = (currentForecast ? currentForecast.trueValue : 0)

         //get current balance
        DepotPosition currentDepotPosition = DepotPosition.withCriteria (uniqueResult: true){
          maxResults(1)
          eq('person', person)
          eq("latest", true)
        }
        BigDecimal realizedValue = (currentDepotPosition ? currentDepotPosition.balance : 0)

        // forecast achieved?
        if(trueValue != realizedValue){
          def deltaValue = realizedValue - trueValue

          def balancingCosts
          // not enough
          if(deltaValue < 0){
            balancingCosts = -deltaValue * (maxProductPrice.getPrice() + product.competition.balancingCostUnder)
          }
          //too much
          else{
            balancingCosts = +deltaValue * product.competition.balancingCostOver
          }

          //write cash delta
          def currentCashPosition = CashPosition.findByPersonAndLatest(person, true)
          def currentCashBalance = 0
          if (currentCashPosition) {
            currentCashBalance = currentCashPosition.getBalance()
            currentCashPosition.latest = false
            currentCashPosition.save()
          }
          CashPosition cashPositionBuyer = new CashPosition(person: person, change: -balancingCosts, transactionID: 0, balance: (currentCashBalance - balancingCosts), description: "balancing power")
          if (!cashPositionBuyer.save()) throw new SettlementException("Failed to create CashPosition for balancing")
          queueOutputService.sendCash(cashPositionBuyer)

          //clear product from depot
          if(currentDepotPosition){
            currentDepotPosition.latest = false
            currentDepotPosition.save()
          }
          DepotPosition balancingDepotPosition = new DepotPosition(person: person, product: product, change: -realizedValue, transactionID: 0, balance: 0, description: "balancing power")
          if (!balancingDepotPosition.save()) throw new SettlementException("Failed to create DepotPosition for balancing")
          queueOutputService.sendDepot(balancingDepotPosition)
        }
      }
    }
}
