/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.converters.*

class QueueOutputService {

  boolean transactional = false

  /*
  * Private notifications sent to the user specific queue
  */

  def sendOrderstatus(Shout shout) throws ShoutNotFoundException {
    if (!shout) throw new ShoutNotFoundException("Cannot send shout '${shout}' to jms.")
    sendToPersonalQueue(shout.person.toQueueName(), shout.toShoutNotificationXml())
    return null
  }

  def sendCash(CashPosition cashPosition) throws CashPositionNotFoundException {
    if (!cashPosition) throw new CashPositionNotFoundException("Cannot send cashPosition ${cashPosition} to jms.")
    sendToPersonalQueue(cashPosition.person.toQueueName(), cashPosition.toCashPositionNotificationXml())
    return null
  }

  def sendDepot(DepotPosition depotPosition) throws DepotPositionNotFoundException {
    if (!depotPosition) throw new DepotPositionNotFoundException("Cannot send depotPosition ${depotPosition} to jms.")
    sendToPersonalQueue(depotPosition.person.toQueueName(), depotPosition.toDepotPositionNotificationXml())
    return null
  }

  def sendForecast(Forecast forecast) throws ForecastNotFoundException {
    if (!forecast) throw new ForecastNotFoundException("Cannot send forecast ${forecast} to jms.")
    sendToPersonalQueue(forecast.person.toQueueName(), forecast.toForecastNotificationXml())
    return null
  }

  def sendToPersonalQueue(queueName, xmlString) {
    try {
      sendQueueJMSMessage(queueName, xmlString)
    } catch (Exception e) {
      log.error("Failed to send Notification '${xmlString}' to queue ${queueName}.", e)
    }
    return null
  }

  /*
  * Public information sent to all private queues
  */

  def sendCompetitionNotification(Competition competition, ActionType actionType) throws CompetitionNotFoundException {
    if (!competition) throw new CompetitionNotFoundException("Cannot send system notification for competition ${competition} to jms.")
    def competitionNotification = competition.toCompetitionNotification()
    if (actionType) competitionNotification.action = actionType
    String xmlString = competitionNotification as XML
    sendToQueues(xmlString)
    log.info "Competition announcement sent: ${xmlString}"
    return null

  }

  //send a product product notification of current product status ("Product opened" | "Product closed") and properties of respective product object

  def sendProductNotification(Product product, ActionType actionType) throws ProductNotFoundException {
    if (!product) throw new ProductNotFoundException("Cannot send product notification for product ${product} to jms.")
    def productNotification = product.toProductNotification()
    if (actionType) productNotification.action = actionType
    String xmlString = productNotification as XML
    sendToQueues(xmlString)
    log.info "Product announcement sent: ${xmlString}"
    return null
  }

  /*
  * send quote, trade and orderbook notifications with properties of respective objects
  */

  def sendQuoteNotification(TransactionLog quote) throws QuoteNotFoundException {
    if (!quote) throw new QuoteNotFoundException("Cannot send quote ${quote} to jms.")
    String xmlString = quote.toQuoteLogNotificationXml()
    sendToQueues(xmlString)
    log.info "Quote announcement sent: ${xmlString}"
    return null
  }

  def sendTradeNotification(TransactionLog trade) throws TradeNotFoundException {
    if (!trade) throw new TradeNotFoundException("Cannot send quote ${trade} to jms.")
    String xmlString = trade.toTradeLogNotificationXml()
    sendToQueues(xmlString)
    log.info "Trade announcement sent: ${xmlString}"
    return null
  }

  def sendOrderbookNotification(Orderbook orderbook) throws OrderbookException {
    if (!orderbook) throw new OrderbookException("Cannot send Orderbook ${orderbook} to jms.")
    String xmlString = orderbook.toOrderbookNotificationXml()
    sendToQueues(xmlString)
    log.info "Ordebook announcement sent: ${xmlString}"
    return null
  }

  def sendToQueues(String xmlString) {
    String queueNames = Competition.findByCurrent(true).queueNames
    try {
      sendQueueJMSMessage(queueNames, xmlString)
    } catch (Exception e) {
      log.error("Failed to send object '${object}' to topic ${topicName}.", e)
    }
    return null
  }

}
