/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package tacenergy

import edu.kit.iism.cdamarket.Competition
import edu.kit.iism.cdamarket.CompetitionStatus
import edu.kit.iism.cdamarket.Person
import edu.kit.iism.cdamarket.Product
import edu.kit.iism.cdamarket.TransactionLog

class CompetitionTagLib {

  def ifCompetitionRunning = { attrs, body ->
		if (Competition.findByCompetitionStatus(CompetitionStatus.Running)) {
			out << body()
		}
	}

  def ifCompetitionNotRunning = { attrs, body ->
		if (!Competition.findByCompetitionStatus(CompetitionStatus.Running)) {
			out << body()
		}
	}

  def running = {attrs, body ->
    def competition = Competition.findByCompetitionStatus(CompetitionStatus.Running)
    if (!competition) {
      out << render(template: "noCurrentCompetition")
    } else {
			def model = [:]
			model.competition = competition
			model.personList = Person.findAllByCompetition(competition)
			model.productList = Product.findAllByEnabledAndCompetition(true, competition)
      model.transactionLogList = TransactionLog.findAllByCompetition(competition, [sort: 'dateCreated', order: 'desc', max: 5])
      out << render(template: "currentCompetition", model: model)
    }
  }


  def participantReadyList = {attrs, body ->
    if (attrs.competition) {
      Person.findAllByCompetition(attrs.competition)?.each {person ->
        out << render(template: "participantReadyStatus", model: [person: person])
      }
    }
  }

  def nextCompetitions = {attrs, body ->
    def competitions = Competition.findAllByCompetitionStatus(CompetitionStatus.Initialized)
    if (!competitions) {
      out << render(template: "noNextCompetitions")
    } else {
      out << render(template: "nextCompetitions", model: [competitionList: competitions])
    }
  }
}
