/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket.profile


/**
 * Provides base functionality for managing BDEW norm load profiles which have
 * datapoints for every hour of a day and distinguish three different seasons
 * over a year (summer, winter, intermediate) as well a three differet day types
 * (Saturday, Sunday, Weekday) 
 *
 * @author Carsten Block
 * @since 0.5
 * @version 1
 * First created: 02.05.2010
 * Last Updated: 02.05.2010
 */
abstract class AbstractThreeSeasonThreeDayGenerator implements Generator {

  Double scaling = 1.0d
  Map<SeasonType, Map> loadProfile

  public AbstractThreeSeasonThreeDayGenerator() {
    initLoadProfile()
  }

  public abstract void initLoadProfile();

  def Double retrieveLoad(Date date) {
    Calendar cal = new GregorianCalendar()
    cal.setTime(date)
    retrieveLoad(cal)
  }

  def Double retrieveLoad(Calendar cal) {
    retrieveLoad(TimeSeriesUtils.getSeasonType(cal), TimeSeriesUtils.getDayType(cal), TimeSeriesUtils.getTimeslot(cal))
  }

  def Double retrieveLoad(SeasonType seasonType, DayType dayType, int timeslot) {
    if (!loadProfile) initLoadProfile()
    double[] currentProfile = loadProfile.get(seasonType).get(dayType)
    return (currentProfile[timeslot] * scaling)
  }


}