/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

/**
 * Provides some utility functions for time series management, e.g.
 * determination of the season type based on a date or a calendar instance
 *
 * @author Carsten Block
 * @since 0.5
 * @version 1
 * First created: 02.05.2010
 * Last Updated: 02.05.2010
 */
package edu.kit.iism.cdamarket.profile

public class TimeSeriesUtils {



  public static int getTimeslot(Date date) {
    Calendar cal = new GregorianCalendar()
    cal.setTime(date)
    getTimeslot(cal)
  }

  public static int getTimeslot(Calendar cal) {
    def hour = cal.get(Calendar.HOUR_OF_DAY)
    def minute = cal.get(Calendar.MINUTE)
    return getTimeslot(hour, minute)
  }


  public static int getTimeslot(int hour, int minute) {
    def quarter = getQuarter(minute)
    return (hour * 4) + quarter
  }


  public static SeasonType getSeasonType(Date date) {
    Calendar cal = new GregorianCalendar()
    cal.setTime(date)
    getSeasonType(cal)

  }

  public static SeasonType getSeasonType(Calendar cal) {

    def dayOfMonth = cal.get(Calendar.DAY_OF_MONTH)
    def month = cal.get(Calendar.MONTH)

    if (month == Calendar.JUNE || month == Calendar.JULY || month == Calendar.AUGUST) {
      return SeasonType.Summer
    } else if (month == Calendar.NOVEMBER || month == Calendar.DECEMBER || month == Calendar.JANUARY || month == Calendar.FEBRUARY || (month == Calendar.MARCH && dayOfMonth <= 20)) {
      return SeasonType.Winter
    } else {
      return SeasonType.Intermediate
    }
  }

  public static DayType getDayType(Date date) {
    Calendar cal = new GregorianCalendar()
    cal.setTime(date)
    getDayType(cal)
  }


  public static DayType getDayType(Calendar cal) {
    if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
      return DayType.Saturday
    } else if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
      return DayType.Sunday
    } else {
      return DayType.Weekday
    }
  }

  public static int getQuarter(Calendar cal) {
    int minute = cal.get(Calendar.MINUTE)
    return getQuarter(minute)
  }

  public static int getQuarter(int minute) {
    int quarter
    if (minute < 15) {
      quarter = 0
    } else if (minute < 30) {
      quarter = 1
    } else if (minute < 45) {
      quarter = 2
    } else {
      quarter = 3
    }
    return quarter
  }
}