/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class ProductIntegrationTests extends GrailsUnitTestCase {

  def competition

  protected void setUp() {
    super.setUp()
    competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    assert(competition.save())
  }

  protected void tearDown() {
    super.tearDown()
  }

  void testComplexDeletionProduct() {

    def myP = new Person(username: "PersonZ", userRealName: 'Personz', passwd:'Secret', email: 'testz@test.com', enabled: true).save()
    def myPr = new Product(name: "ProductA", enabled: true, serialNumber: 1L).save()
    def myDP = new DepotPosition(person: Person.get(myP.id), product: Product.get(myPr.id), change: 100, transactionID: 123L, balance: 0).save()

    def myS = new Shout(person: myP, product: myPr, transactionID: 123L, shoutID: 123L, buySellIndicator: BuySellIndicator.BUY, remainingQuantity: 100).save()

    myPr.addToDepotPositions(myDP)
    myPr.addToShouts(myS)

    def delPr = Product.get(myPr.id)
    delPr.delete()

    def newS = Shout.get(myS.id)
    def newDP = DepotPosition.get(myDP.id)
    def newPr = Product.get(myDP.id)

    println newS
    println newDP
    println newPr
    assertNull "DepotPosition should be deleted", newDP
    assertNull "Shout should be deleted", newS
  }

  void testCompetitionSetAutomaticallyUponProductCreation() {
    def myPr = new Product(name: "ProductA", enabled: true, serialNumber: 1L).save()
    assertNotNull(myPr)
    assertEquals(competition, myPr.competition)
  }
}