/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

class ProductManagementServiceIntegrationTests extends GroovyTestCase {

  def productManagementService

  void setUp() {
    super.setUp()
    assert (productManagementService)
  }

  void testProductInitialization() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current=true
    competition.save()

    def products = Product.findAllByCompetition(competition)
    assertEquals(0, products.size())
    productManagementService.initProducts(competition)
    products = Product.findAllByCompetition(competition)
    assertEquals(5, products.size())
    assertEquals(false, products[0].enabled)
    assertEquals(true, products[0].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[1].enabled)
    assertEquals(false, products[1].current)
    assertEquals(competition, products[1].competition)
    assertEquals(false, products[2].enabled)
    assertEquals(false, products[2].current)
    assertEquals(competition, products[2].competition)
    assertEquals(false, products[3].enabled)
    assertEquals(false, products[3].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[4].enabled)
    assertEquals(false, products[4].current)
    assertEquals(competition, products[4].competition)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 00).getTime(), products[0].startDateTime)
    assertEquals(new GregorianCalendar(2006, 05, 05, 15, 14, 59).getTime(), products[4].endDateTime)
  }

  void testGetCurrentProduct() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    productManagementService.initProducts(competition)
    def products = Product.findAllByCompetition(competition)
    assertEquals(5, products.size())
    assertEquals(true, products[0].current)
    assertEquals(products[0], productManagementService.getCurrentProduct(competition))
    try {
      productManagementService.getCurrentProduct(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  void testOpenFiveProductsOverallThreeInitiallyOpenONeDeactivatedAhead() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    def products = Product.findAllByCompetition(competition)
    assertEquals(0, products.size())

    productManagementService.initProducts(competition)
    productManagementService.openInitialProducts(competition)

    assertEquals(3, Product.findAllByCompetitionAndEnabled(competition, true).size())

    products = Product.findAllByCompetition(competition)
    assertEquals(5, products.size())

    assertEquals(false, products[0].enabled)
    assertEquals(true, products[0].current)
    assertEquals(competition, products[0].competition)
    assertEquals(true, products[1].enabled)
    assertEquals(false, products[1].current)
    assertEquals(competition, products[1].competition)
    assertEquals(true, products[2].enabled)
    assertEquals(false, products[2].current)
    assertEquals(competition, products[2].competition)
    assertEquals(true, products[3].enabled)
    assertEquals(false, products[3].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[4].enabled)
    assertEquals(false, products[4].current)
    assertEquals(competition, products[4].competition)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 00).getTime(), products[0].startDateTime)
    assertEquals(new GregorianCalendar(2006, 05, 05, 15, 14, 59).getTime(), products[4].endDateTime)


  }

  void testOpenTwoInitialWithTwoDeactivatedProductsAhead() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 2
    competition.deactivateTimeslotsAhead = 2
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    def products = Product.findAllByCompetition(competition)
    assertEquals(0, products.size())

    productManagementService.initProducts(competition)
    productManagementService.openInitialProducts(competition)

    assertEquals(2, Product.findAllByCompetitionAndEnabled(competition, true).size())

    products = Product.findAllByCompetition(competition)
    assertEquals(5, products.size())

    assertEquals(false, products[0].enabled)
    assertEquals(true, products[0].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[1].enabled)
    assertEquals(false, products[1].current)
    assertEquals(competition, products[1].competition)
    assertEquals(true, products[2].enabled)
    assertEquals(false, products[2].current)
    assertEquals(competition, products[2].competition)
    assertEquals(true, products[3].enabled)
    assertEquals(false, products[3].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[4].enabled)
    assertEquals(false, products[4].current)
    assertEquals(competition, products[4].competition)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 00).getTime(), products[0].startDateTime)
    assertEquals(new GregorianCalendar(2006, 05, 05, 15, 14, 59).getTime(), products[4].endDateTime)
  }

  void testOpenTwoProductsNoDeactivateAheadOneActiveProduct() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 2
    competition.timeslotLength = 15
    competition.timeslotsOpen = 1
    competition.deactivateTimeslotsAhead = 0
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    def products = Product.findAllByCompetition(competition)
    assertEquals(0, products.size())

    productManagementService.initProducts(competition)
    productManagementService.openInitialProducts(competition)

    assertEquals(1, Product.findAllByCompetitionAndEnabled(competition, true).size())

    products = Product.findAllByCompetition(competition)
    assertEquals(2, products.size())

    assertEquals(true, products[0].enabled)
    assertEquals(true, products[0].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[1].enabled)
    assertEquals(false, products[1].current)
    assertEquals(competition, products[1].competition)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 00).getTime(), products[0].startDateTime)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 29, 59).getTime(), products[1].endDateTime)
  }


  void testShiftProducts() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 6
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00, 00).getTime()
    competition.current = true
    competition.save()

    def products = Product.findAllByCompetition(competition)
    assertEquals(0, products.size())

    productManagementService.initProducts(competition)
    products = Product.findAllByCompetition(competition)
    assertEquals(6, products.size())
    productManagementService.openInitialProducts(competition)
    products = Product.findAllByCompetition(competition)
    assertEquals(6, products.size())
    assertEquals(3, Product.findAllByCompetitionAndEnabled(competition, true).size())

    //First Product shift
    productManagementService.shiftProducts(competition)

    products = Product.list()
    sleep(1000)
    products = Product.findAllByCompetition(competition)
    assertEquals(6, products.size())

    assertEquals(false, products[0].enabled)
    assertEquals(false, products[0].current)
    assertEquals(competition, products[0].competition)
    assertEquals(false, products[1].enabled)
    assertEquals(true, products[1].current)
    assertEquals(competition, products[1].competition)
    assertEquals(true, products[2].enabled)
    assertEquals(false, products[2].current)
    assertEquals(competition, products[2].competition)
    assertEquals(true, products[3].enabled)
    assertEquals(false, products[3].current)
    assertEquals(competition, products[0].competition)
    assertEquals(true, products[4].enabled)
    assertEquals(false, products[4].current)
    assertEquals(competition, products[4].competition)
    assertEquals(false, products[5].enabled)
    assertEquals(false, products[5].current)
    assertEquals(competition, products[5].competition)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 00).getTime(), products[0].startDateTime)
    assertEquals(new GregorianCalendar(2006, 05, 05, 15, 29, 59).getTime(), products[5].endDateTime)

    //Second Product shift
    productManagementService.shiftProducts()
    products = Product.findAllByCompetition(competition)
    assertEquals(new GregorianCalendar(2006, 05, 05, 14, 00).getTime(), products[0].startDateTime)
    assertEquals(6, products.size())
    assertEquals(false, products[0].enabled)
    assertEquals(false, products[0].current)
    assertEquals(false, products[1].enabled)
    assertEquals(false, products[1].current)
    assertEquals(false, products[2].enabled)
    assertEquals(true, products[2].current)
    assertEquals(true, products[3].enabled)
    assertEquals(false, products[3].current)
    assertEquals(true, products[4].enabled)
    assertEquals(false, products[4].current)
    assertEquals(true, products[5].enabled)
    assertEquals(false, products[5].current)
  }

  void testInitProductsNullValues() {
    try {
      productManagementService.initProducts(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  void testOpenInitialProductsNullValues() {
    try {
      productManagementService.openInitialProducts(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }

  }

  void testOpenInitialProductsNoCurrentProduct() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 6
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00, 00).getTime()
    competition.current = true
    competition.save()
    productManagementService.initProducts(competition)
    def currentProduct = Product.findByCompetitionAndCurrent(competition, true)
    currentProduct.current = false
    currentProduct.save()

    try {
      productManagementService.openInitialProducts(competition)
    } catch (Exception e) {
      assertTrue(e instanceof NoCurrentProductException)
    }
  }

  void testOpenInitialProductsNoProductsFound() {

    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 6
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00, 00).getTime()
    competition.current = true
    competition.save()
    productManagementService.initProducts(competition)

    Product.findAllByCompetition(competition)?.each {
      it.delete()
    }
    assertEquals(0, Product.findAllByCompetition(competition)?.size())
    try {
      productManagementService.openInitialProducts(competition)
    } catch (Exception e) {
      assertTrue(e instanceof OpenInitialProductsException)
    }
  }

  void testGetOpenProductsNull() {
    try {
      productManagementService.getOpenProducts(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }

    try {
      productManagementService.getOpenProducts(null, null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  void testGetOpenProducts() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 2
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    def products = Product.findAllByCompetition(competition)
    assertEquals(0, products.size())

    productManagementService.initProducts(competition)
    productManagementService.openInitialProducts(competition)

    products = productManagementService.getOpenProducts(competition)

    def firstProduct = Product.findByStartDateTime(new GregorianCalendar(2006, 05, 05, 14, 30).getTime())
    def secondProduct = Product.findByStartDateTime(new GregorianCalendar(2006, 05, 05, 14, 45).getTime())
    def thirdProduct = Product.findByStartDateTime(new GregorianCalendar(2006, 05, 05, 15, 00).getTime())

    assertEquals(3, products.size())
    assertEquals(firstProduct.id, products[0].id)
    assertEquals(secondProduct.id, products[1].id)
    assertEquals(thirdProduct.id, products[2].id)

    def limitedOpenProducts = productManagementService.getOpenProducts(competition, 2)
    assertEquals(2, limitedOpenProducts.size())
    assertEquals(firstProduct.id, limitedOpenProducts[0].id)
    assertEquals(secondProduct.id, limitedOpenProducts[1].id)
  }

  void testGetAllProductsWithNullCompetition() {
    try {
      productManagementService.getAllProducts(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  void testSwitchProductEnableNullValues() {
    try {
      productManagementService.switchProductEnabled(null, new Product(), false)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }

    try {
      productManagementService.switchProductEnabled(new Competition(), null, false)
    } catch (Exception e) {
      assertTrue(e instanceof ProductNotFoundException)
    }

    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 2
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    productManagementService.initProducts(competition)

    def product = Product.findByCompetition(competition)
    assertNotNull(product)

    product.enabled = true
    product.save()
    productManagementService.switchProductEnabled(competition, product, null)
    assertEquals(false, product.enabled)

    product.enabled = true
    product.save()
    productManagementService.switchProductEnabled(competition, product, false)
    assertEquals(false, product.enabled)

    productManagementService.switchProductEnabled(competition, product, true)
    assertEquals(true, product.enabled)
  }

  void testDeactivateOldestProductNull() {
    try {
      productManagementService.deactivateOldestProduct(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  void testDeactivateOldestProductNoActiveProducts() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 2
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    productManagementService.initProducts(competition)
    try {
      productManagementService.deactivateOldestProduct(competition)
    } catch (Exception e) {
      assertTrue(e instanceof NoActiveProductException)
    }
  }

  void testDeactivateOldestProduct() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 2
    competition.timeslotLength = 15
    competition.timeslotsOpen = 1
    competition.deactivateTimeslotsAhead = 1
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    productManagementService.initProducts(competition)
    productManagementService.openInitialProducts(competition)

    def products = Product.findAllByCompetition(competition)
    assertEquals(false, products[0].enabled)
    assertEquals(true, products[1].enabled)
    productManagementService.deactivateOldestProduct(competition)
    assertEquals(false, products[1].enabled)
  }

  void testActivateNewProductCompetitionNull() {
    try {
      productManagementService.activateNewProduct(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  void testActivateNewProductNoActiveProducts() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 5
    competition.timeslotLength = 15
    competition.timeslotsOpen = 3
    competition.deactivateTimeslotsAhead = 2
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    productManagementService.initProducts(competition)
    try {
      productManagementService.activateNewProduct(competition)
    } catch (Exception e) {
      assertTrue(e instanceof NoActiveProductException)
    }
  }

  void testActivateNewProduct() {
    def competition = new Competition(name: "test", enabled: true)
    competition.timeslotsOverall = 2
    competition.timeslotLength = 15
    competition.timeslotsOpen = 1
    competition.deactivateTimeslotsAhead = 0
    competition.simulationStartTime = new GregorianCalendar(2006, 05, 05, 14, 00).getTime()
    competition.current = true
    competition.save()

    productManagementService.initProducts(competition)
    productManagementService.openInitialProducts(competition)

    def products = Product.findAllByCompetition(competition)
    assertEquals(true, products[0].enabled)
    assertEquals(false, products[1].enabled)
    assertEquals(products[1].id, productManagementService.activateNewProduct(competition).id)
    assertEquals(true, products[1].enabled)
    assertNull(productManagementService.activateNewProduct(competition)) //No more products after products[1]
  }

  void testAdvanceCurrentProductNull() {
    try {
      productManagementService.advanceCurrentProduct(null)
    } catch (Exception e) {
      assertTrue(e instanceof CompetitionNotFoundException)
    }
  }

  

}
