/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class CashPositionTests extends GrailsUnitTestCase {
  protected void setUp() {
    super.setUp()
  }

  protected void tearDown() {
    super.tearDown()
  }

   //test if toString method returns correct pattern

  void testToString() {
    mockDomain(CashPosition)
    def myCP = new CashPosition(person: new Person(username: "PersonA"), change: 100, description: "Transaction1", transactionID: 123L)
    assertToString myCP, "PersonA-100-Transaction1-${myCP.dateCreated}"
    println myCP

  }

  // Create

  void testNullablePerson() {
    mockDomain(CashPosition);
    def myCP = new CashPosition(change: 100)

    assertFalse 'validation should have failed', myCP.validate()
    assertEquals "nullable", myCP.errors.person
    println "Error reason: ${myCP.errors.person}"
  }

  void testNullableChange() {
    mockDomain(CashPosition);
    def myCP = new CashPosition(person: new Person())

    assertFalse 'validation should have failed', myCP.validate()
    assertEquals "nullable", myCP.errors.change
    println "Error reason: ${myCP.errors.change}"
  }

  // test if dateCreated is set correctly

  void testDateCreated() {
    mockDomain(CashPosition);
    def myCP = new CashPosition(person: new Person(), change: 100)
    myCP.save()
    assertNotNull myCP.dateCreated
    println "DateCreated: ${myCP.dateCreated}"
  }

  // Read: test if get methods work correctly

  void testGetPerson() {
    mockDomain(CashPosition)

    def testPerson = new Person(username: "PersonA")
    def myCP = new CashPosition(person: testPerson, change: 100, transactionID: 123L, balance: 0).save();
    def newCP = CashPosition.get(myCP.id)
    assertEquals newCP.getPerson(), testPerson
  }

  void testGetChange() {
    mockDomain(CashPosition)
    def myCP = new CashPosition(person: new Person(), change: 100, transactionID: 123L, balance: 0).save()
    def newCP = CashPosition.get(myCP.id)
    assertEquals newCP.change, 100
  }

  void testGetDescription() {
    mockDomain(CashPosition)
    String desc = "This is a Description"
    def myCP = new CashPosition(person: new Person(), description: desc, change: 100, transactionID: 123L, balance: 0).save()
    def newCP = CashPosition.get(myCP.id)
    assertEquals newCP.getDescription(), desc
  }

  void testGetDateCreated() {
    mockDomain(CashPosition)
    def myCP = new CashPosition(person: new Person(), change: 100, transactionID: 123L, balance: 0).save()
    def newCP = CashPosition.get(myCP.id)
    assertEquals newCP.getDateCreated(), myCP.getDateCreated()
  }

// Update

  void testUpdateChange() {
    mockDomain(CashPosition)
    def myCP = new CashPosition(person: new Person(), change: 100, transactionID: 123L, balance: 0).save()
    def newCP = CashPosition.get(myCP.id)
    newCP.setChange(200)
    assertEquals myCP.change, 200
  }

  void testUpdateDescription() {
    mockDomain(CashPosition)
    def myCP = new CashPosition(person: new Person(), change: 100, description: "First Text", transactionID: 123L, balance: 0).save()
    def newCP = CashPosition.get(myCP.id)
    newCP.setDescription("Second Text")
    assertEquals myCP.getDescription(), "Second Text"
  }

  void testUpdatePerson() {
    mockDomain(CashPosition)
    def myCP = new CashPosition(person: new Person(username: "PersonA"), change: 100, transactionID: 123L, balance: 0).save()
    println "First Person: ${myCP.getPerson()}"
    Person pB = new Person(username: "PersonB")
    def newCP = CashPosition.get(myCP.id)
    newCP.setPerson(pB)

    assertToString myCP.getPerson(), "PersonB"
    println "Second Person: ${myCP.getPerson()}"
  }

  //Delete
  //test if CashPosition is correctly deleted from database

  void testDeletion() {
    mockDomain(CashPosition)
    def testPerson = new Person(username: "PersonA")

    def myCP = new CashPosition(person: testPerson, change: 100, transactionID: 123L, balance: 0).save();
    def delCP = CashPosition.findByPerson(testPerson)
    assertNotNull "get method should return object that is not null", delCP
    delCP.delete(flush: true)

    def newCP = CashPosition.get(myCP.id)
    assertNull "myCP should be deleted from database", newCP
    println myCP
    println delCP
    println newCP
  }


}
