/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class DepotPositionTests extends GrailsUnitTestCase {
  protected void setUp() {
    super.setUp()
  }

  protected void tearDown() {
    super.tearDown()
  }

  //test if toString method returns correct pattern
  void testToString() {
    mockDomain(DepotPosition)
    def myDP = new DepotPosition(person: new Person(username: "PersonA"), product: new Product(name: "ProductA"), change: 100, description: "Transaction1", balance: 0)
    assertToString myDP, "PersonA-ProductA-100-Transaction1-${myDP.dateCreated}"
    println myDP
  }

  // Create
  void testNullablePerson() {
    mockDomain(DepotPosition);
    def myDP = new DepotPosition(change: 100, balance: 0)

    assertFalse 'validation should have failed', myDP.validate()
    assertEquals "nullable", myDP.errors.person
    println "Error reason: ${myDP.errors.person}"
  }

  //TODO NullableQuantity

  // test if dateCreated is set correctly
  void testDateCreated() {
    mockDomain(DepotPosition);
    def myDP = new DepotPosition(person: new Person(), product: new Product(), change: 100, transactionID: 123L , balance: 0).save()
    assertNotNull myDP.dateCreated
    println "DateCreated: ${myDP.dateCreated}"
  }

  // Read: test if get methods work correctly

  void testGetPerson() {
    mockDomain(DepotPosition)

    def testPerson = new Person(username: "PersonA")
    def myDP = new DepotPosition(person: testPerson, product: new Product(), change: 100, transactionID: 123L, balance: 0).save();
    def newDP = DepotPosition.get(myDP.id)
    assertEquals newDP.getPerson(), testPerson
  }

  void testGetSize() {
    mockDomain(DepotPosition)
    def myDP = new DepotPosition(person: new Person(), product: new Product(), change: 100, transactionID: 123L, balance: 0).save()
    def newDP = DepotPosition.get(myDP.id)
    assertEquals newDP.getChange(), 100
  }

  void testGetDescription() {
    mockDomain(DepotPosition)
    String desc = "This is a Description"
    def myDP = new DepotPosition(person: new Person(), product: new Product(), description: desc, change: 100, transactionID: 123L, balance: 0).save()
    def newDP = DepotPosition.get(myDP.id)
    assertEquals newDP.getDescription(), desc
  }

  void testGetDateCreated() {
    mockDomain(DepotPosition)
    def myDP = new DepotPosition(person: new Person(), product: new Product(), change: 100, transactionID: 123L, balance: 0).save()
    def newDP = DepotPosition.get(myDP.id)
    assertEquals newDP.getDateCreated(), myDP.getDateCreated()
  }

  // Update

  void testUpdatePerson() {
    mockDomain(DepotPosition)
    mockDomain(Person)
    def myDP = new DepotPosition(person: new Person(username: "PersonA"), product: new Product(), change: 100, transactionID: 123L, balance: 0).save()
    println "First Person: ${myDP.getPerson()}"
    Person pB = new Person(username: "PersonB")
    def newDP = DepotPosition.get(myDP.id)
    newDP.setPerson(pB)

    assertToString myDP.getPerson(), "PersonB"
    println "Second Person: ${myDP.getPerson()}"
  }

  void testUpdateSize() {
    mockDomain(DepotPosition)
    def myDP = new DepotPosition(person: new Person(), product: new Product(), change: 100, transactionID: 123L, balance: 0).save()
    def newDP = DepotPosition.get(myDP.id)
    newDP.setChange(200)
    assertEquals myDP.change, 200
  }

  void testUpdateDescription() {
    mockDomain(DepotPosition)
    def myDP = new DepotPosition(person: new Person(), product: new Product(), change: 100, description: "First Text", transactionID: 123L, balance: 0).save()
    def newDP = DepotPosition.get(myDP.id)
    newDP.setDescription("Second Text")
    assertEquals myDP.getDescription(), "Second Text"
  }

  // Delete

  //test if DepotPosition is correctly deleted from database
  void testDeletionPerson() {
    mockDomain(DepotPosition)
    def testPerson = new Person(username: "PersonA")

    def myDP = new DepotPosition(person: testPerson, product: new Product(), change: 100, transactionID: 123L, balance: 0).save();
    def delDP = DepotPosition.findByPerson(testPerson)
    assertNotNull "get method should return object that is not null", delDP

    delDP.delete(flush: true)

    def newDP = DepotPosition.get(myDP.id)
    assertNull "myDP should be deleted from database", newDP
    println myDP
    println delDP
    println newDP
  }

  void testDeletionProduct() {
    mockDomain(DepotPosition)
    def testProduct = new Product(name: "PersonA")

    def myDP = new DepotPosition(person: new Person(), product: testProduct, change: 100, transactionID: 123L, balance: 0).save();
    def delDP = DepotPosition.findByProduct(testProduct)
    assertNotNull "get method should return object that is not null", delDP

    delDP.delete(flush: true)

    def newDP = DepotPosition.get(myDP.id)
    assertNull "myDP should be deleted from database", newDP
    println myDP
    println delDP
    println newDP
  }

}
