/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class PersonTests extends GrailsUnitTestCase {
  protected void setUp() {
    super.setUp()
  }

  protected void tearDown() {
    super.tearDown()
  }

  // Create

  void testNullableName() {
    mockDomain(Person)
    def myP = new Person()

    assertFalse myP.validate()
    assertEquals "nullable", myP.errors.username
  }

  // Read
  void testGetName() {
    mockDomain(Person)
    def myP = new Person(username: "PersonA", userRealName: 'PersonA', passwd:'Secret', email: 'test@test.com', enabled: true).save()
    def newP = Person.get(myP.id)
    assertEquals newP.username, myP.username
  }

  void testGetActivated() {
    mockDomain(Person)
    def myP = new Person(username: "PersonA", userRealName: 'PersonA', passwd:'Secret', email: 'test@test.com', enabled: true).save()
    def newP = Person.get(myP.id)

    assert newP.enabled
  }

  // Update
  void testUpdateName() {
    mockDomain(Person)
    mockDomain(Shout)
    
    def myP = new Person(username: "PersonA", userRealName: 'PersonA', passwd:'Secret', email: 'test@test.com', enabled: true).save()
    def newP = Person.get(myP.id)
    newP.username = "PersonB"

    assertEquals myP.username, "PersonB"
  }

  /*
   //Not working in the unit test environment as the enabled setter is overwritten to use Shout class, which is not injected --> See integration tests
  void testUpdateActivated() {
    mockDomain(Person)
    def myP = new Person(username: "PersonA", userRealName: 'PersonA', passwd:'Secret', email: 'test@test.com', enabled: false).save()
    def newP = Person.get(myP.id)
    newP.enabled = true

    assert myP.enabled

    newP.enabled = false
    assertFalse myP.enabled
  }
  */


  void testSimpleDeletion() {
    mockDomain(Person)
    def myP = new Person(username: "PersonA", userRealName: 'PersonA', passwd:'Secret', email: 'test@test.com', enabled: true).save()
    def delP = Person.get(myP.id)
    delP.delete(flush: true)

    def newP = Person.get(myP.id)
    assertNull "myP should be deleted from database", newP
  }
 

  
}
