/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class ProductTests extends GrailsUnitTestCase {
    protected void setUp() {
        super.setUp()
    }

    protected void tearDown() {
        super.tearDown()
    }

    // Create
    void testNullableName() {
      mockDomain(Product)
      def myP = new Product()

      assertFalse 'validation should have failed', myP.validate()
      assertEquals "nullable", myP.errors.name
    }

    void testAllAttributes(){
      mockDomain(Product)
      def myP = new Product(name: "Test Product A", enabled: true, serialNumber: 1L).save()
      myP.enabled = true
      myP.startDateTime = new Date()
      myP.endDateTime = new Date()

      assertTrue 'validation failed', myP.validate()
    }

    // Read
    void testGetName() {
      mockDomain(Product)
      def productName = new String("Test Product A")

      def myP = new Product(name: productName, enabled: true, serialNumber: 1L).save()
      def newP = Product.findByName(productName)

      assertEquals newP.name, productName
    }

    // Update
    void testUpdateName() {
      mockDomain(Product)
      def productNameA = new String("Test Product A")
      def productNameB = new String("Test Product B")

      def myP = new Product(name: productNameA, enabled: true, serialNumber: 1L).save()
      def newP = Product.findByName(productNameA)
      newP.setName(productNameB)

      assertEquals myP.name, productNameB
    }

    //Delete
    void testSimpleDeletion() {
      mockDomain(Product)
      def productName = new String("Test Product A")

      def myP = new Product(name: productName, enabled: true, serialNumber: 1L).save()
      def delP = Product.findByName(productName)

      delP.delete(flush: true)

      def newP = Product.findByName(productName)
      assertNull "Product should be deleted from database", newP
    }
}
