/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class ShoutTests extends GrailsUnitTestCase {

    protected void setUp() {
        super.setUp()
    }

    protected void tearDown() {
        super.tearDown()
    }

  void testDeactivatedPerson() {
    mockDomain(Shout);
    //Right way
    def myS1 = new Shout()
    myS1.transactionID = 123L
    myS1.shoutID = 1L
    myS1.person =  new Person(username: "PersonA", enabled: true)
    myS1.product = new Product(name: "ProductA", enabled: true)
    myS1.buySellIndicator = BuySellIndicator.BUY
    myS1.remainingQuantity = 100
    myS1.limitPrice = 1.0
    myS1.orderType = OrderType.LIMIT
    assertTrue 'validation should not have failed', myS1.validate()

    //Wrong way: person deactivated
    def myS2 = new Shout()
    myS2.transactionID = 123L
    myS2.shoutID = 1L
    myS2.person =  new Person(username: "PersonA", enabled: false)
    myS2.product = new Product(name: "ProductA", enabled: true)
    myS2.buySellIndicator = BuySellIndicator.BUY
    myS2.remainingQuantity = 100
    myS2.orderType = OrderType.LIMIT
    assertFalse 'validation should have failed', myS2.validate()
    assertEquals "validator", myS2.errors.person
  }

    void testDeactivatedProduct() {
    mockDomain(Shout);
    //Right way
    def myS1 = new Shout()
    myS1.transactionID = 123L
    myS1.shoutID = 1L
    myS1.person =  new Person(username: "PersonA", enabled: true)
    myS1.product = new Product(name: "ProductA", enabled: true)
    myS1.buySellIndicator = BuySellIndicator.BUY
    myS1.remainingQuantity = 100
    myS1.limitPrice = 1.0
    myS1.orderType = OrderType.LIMIT
    assertTrue 'validation should not have failed', myS1.validate()

    //Wrong way: product deactivated
    def myS2 = new Shout()
    myS2.transactionID = 123L
    myS2.shoutID = 1L
    myS2.person =  new Person(username: "PersonA", enabled: true)
    myS2.product = new Product(name: "ProductA", enabled: false)
    myS2.buySellIndicator = BuySellIndicator.BUY
    myS2.remainingQuantity = 100
    myS2.orderType = OrderType.LIMIT
    assertFalse 'validation should have failed', myS2.validate()
    assertEquals "validator", myS2.errors.product
  }

  void testCreateLimitOrder(){
    mockDomain(Shout);

    //Right way
    def myS1 = new Shout()
    myS1.transactionID = 123L
    myS1.person =  new Person(username: "PersonA", enabled: true)
    myS1.product = new Product(name: "ProductA", enabled: true)
    myS1.buySellIndicator = BuySellIndicator.BUY
    myS1.remainingQuantity = 100
    myS1.limitPrice = 1.0  //<- limit limitPrice
    myS1.orderType = OrderType.LIMIT
    myS1.shoutID = 1L
    myS1.transactionID = 1L
    myS1.save()
    assertTrue 'validation should not have failed', myS1.validate()

    //Wrong way     
    def myS2 = new Shout()
    myS2.transactionID = 123L
    myS2.person =  new Person(username: "PersonA", enabled: true)
    myS2.product = new Product(name: "ProductA", enabled: true)
    myS2.buySellIndicator = BuySellIndicator.BUY
    myS2.remainingQuantity = 100
    //no limit limitPrice
    myS2.orderType = OrderType.LIMIT
    myS1.shoutID = 1L
    myS1.transactionID = 1L
    myS2.save()
    assertFalse 'validation should have failed', myS2.validate()
  }

  void testCreateMarketOrder(){
    mockDomain(Shout);

    //Right way
    def myS1 = new Shout()
    myS1.transactionID = 123L
    myS1.person =  new Person(username: "PersonA", enabled: true)
    myS1.product = new Product(name: "ProductA", enabled: true)
    myS1.buySellIndicator = BuySellIndicator.BUY
    myS1.remainingQuantity = 100
    //no limit limitPrice
    myS1.orderType = OrderType.MARKET
    myS1.shoutID = 1L
    myS1.transactionID = 1L
    myS1.save()
    assertTrue 'validation should not have failed', myS1.validate()

    //Wrong way
    def myS2 = new Shout()
    myS2.transactionID = 123L
    myS2.person =  new Person(username: "PersonA", enabled: true)
    myS2.product = new Product(name: "ProductA", enabled: true)
    myS2.buySellIndicator = BuySellIndicator.BUY
    myS2.remainingQuantity = 100
    myS2.limitPrice = 1.0  //<- limit limitPrice
    myS2.orderType = OrderType.MARKET
    myS1.shoutID = 1L
    myS1.transactionID = 1L
    myS2.save()
    assertFalse 'validation should have failed', myS2.validate()
  }

  void testDeleteAllOrderParts(){
    mockDomain(Shout);

    def myS1 = new Shout()
    myS1.person =  new Person(username: "PersonA", enabled: true)
    myS1.product = new Product(name: "ProductA", enabled: true)
    myS1.buySellIndicator = BuySellIndicator.BUY
    myS1.remainingQuantity = 100
    myS1.transactionID = 123L
    myS1.shoutID = 1L
    myS1.save()
    
    def myS2 = myS1.initModification(ModReasonCode.PARTIALEXECUTION)
    myS2.executionPrice = 10
    myS2.executionQuantity = 10
    myS2.save()
    def myS3 = myS2.initModification(ModReasonCode.EXECUTION)
    myS3.executionPrice = 10
    myS3.executionQuantity = 10
    myS3.save()

    myS3.deleteCompleteOrder()

    def myNewS1 = Shout.get(myS1.id)
    def myNewS2 = Shout.get(myS2.id)
    def myNewS3 = Shout.get(myS3.id)
    assertNull "Not all Shout parts (1) deleted", myNewS1
    assertNull "Not all Shout parts (2) deleted", myNewS2
    assertNull "Not all Shout parts (3) deleted", myNewS3
  }

  void testInitModification() {
    mockDomain(Shout);
    mockDomain(Product);
    mockDomain(Person);

    def myPerson = new Person(username: "PersonA", userRealName: 'PersonA', passwd:'Secret', email: 'test@test.com', enabled: true).save()
    def myProduct = new Product(name: "ProductA", enabled: true, serialNumber: 1L).save()

    def myS = new Shout()
    myS.person = myPerson
    myS.product = myProduct
    myS.transactionID = 123L
    myS.buySellIndicator = BuySellIndicator.BUY
    myS.remainingQuantity = 100
    myS.limitPrice = 1.0
    myS.orderType = OrderType.LIMIT
    myS.shoutID = 1L
    myS.transactionID = 1L
    myS.save()

    Thread.sleep(1000)

    def myNewS = myS.initModification(ModReasonCode.MODIFICATION)

    assertEquals  myNewS.person, myPerson
    assertEquals  myNewS.product, myProduct
    assertEquals  myNewS.buySellIndicator, BuySellIndicator.BUY
    assertEquals  myNewS.remainingQuantity, 100
    assertEquals  myNewS.limitPrice, 1.0
    assertEquals  myNewS.executionQuantity, null
    assertEquals  myNewS.executionPrice, null
    assertEquals  myNewS.orderType, OrderType.LIMIT
    assertEquals  myNewS.dateCreated, myS.dateCreated
    assertNotSame  myNewS.dateMod, myS.dateMod
    assertEquals  myNewS.modReasonCode, ModReasonCode.MODIFICATION
    assertEquals  myNewS.outdated, false
    assertEquals  myNewS.transactionID, null
    assertEquals  myNewS.shoutID, 1L
    assertEquals  myNewS.auctionTradeFlag, AuctionTradeFlag.C

    myNewS.transactionID = 2L
    assertTrue 'cloned shout not valid', myNewS.validate()
  }
}
