/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket

import grails.test.*

class TransactionLogTests extends GrailsUnitTestCase {
  protected void setUp() {
    super.setUp()
  }

  protected void tearDown() {
    super.tearDown()
  }

  private TransactionLog createTL() {
    def myTL = new TransactionLog(product: new Product(name: "ProductA"), transactionID: 123L, type: TransactionType.TRADE, price: 100, quantity: 200, buyerID: new Person(), sellerID: new Person(), buySellIndicator: BuySellIndicator.BUY).save()
    return myTL
  }

  void testToString() {
    def myTL = new TransactionLog(product: new Product(name: "ProductA"), type: TransactionType.TRADE)
    assertToString myTL, "${myTL.dateCreated}-TRADE-ProductA"
    println myTL
  }

  // Create
  void testNullableType() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product())

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "nullable", myTL.errors.type
    println "Error reason: ${myTL.errors.type}"
  }

  void testDateCreated() {
    mockDomain(TransactionLog);
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.TRADE, transactionID: 123L)
    myTL.save()
    println "Error reason: ${myTL.errors.dateCreated}"
    assertNotNull myTL.dateCreated
    println "DateCreated: ${myTL.dateCreated}"
  }

  void testQuoteAndPrice() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.QUOTE, price: 100)

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "validator", myTL.errors.price
    println "Error reason: ${myTL.errors.price}"
  }
  
  void testTradeAndNoPrice() {
	mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.TRADE, buyerID: new Person(), sellerID: new Person(), buySellIndicator: BuySellIndicator.BUY)

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "validator", myTL.errors.price
    println "Error reason: ${myTL.errors.price}"
  
  }

  void testQuoteAndQuantity() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.QUOTE, quantity: 100)

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "validator", myTL.errors.quantity
    println "Error reason: ${myTL.errors.quantity}"
  }

  void testQuoteAndBuyerID() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.QUOTE, buyerID: new Person())

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "validator", myTL.errors.buyerID
    println "Error reason: ${myTL.errors.buyerID}"
  }

  void testQuoteAndSellerID() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.QUOTE, sellerID: new Person())

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "validator", myTL.errors.sellerID
    println "Error reason: ${myTL.errors.sellerID}"
  }

  void testQuoteAndBSInd() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), type: TransactionType.QUOTE, buySellIndicator: BuySellIndicator.BUY)

    assertFalse 'validation should have failed', myTL.validate()
    assertEquals "validator", myTL.errors.buySellIndicator
    println "Error reason: ${myTL.errors.buySellIndicator}"
  }
  
  // Read
  void testGetProduct() {
    mockDomain(TransactionLog)
    def myProduct = new Product(name: "Product A")
    def myTL = new TransactionLog(product: myProduct, transactionID: 123L, type: TransactionType.TRADE, price: 100, quantity: 200, buyerID: new Person(), sellerID: new Person(), buySellIndicator: BuySellIndicator.BUY).save()

    println ("myTL: ${myTL} with ID ${myTL.id}")
    def newTL = TransactionLog.get(myTL.id)
    println ("newTL: ${newTL}")

    assertEquals newTL.product, myProduct
  }

  void testGetType() {
    mockDomain(TransactionLog)
    def myTL = createTL()
    def newTL = TransactionLog.get(myTL.id)

    assertEquals newTL.type, TransactionType.TRADE
  }

  void testGetDateCreated() {
    mockDomain(TransactionLog)
    def myTL = createTL()
    myTL.save()
    def newTL = TransactionLog.get(myTL.id)
    assertEquals newTL.getDateCreated(), myTL.getDateCreated()
  }

  void testTrade() {
    mockDomain(TransactionLog)
    def p1 = new Person(username: "PersonA")
    def p2 = new Person(username: "PersonB")
    def myTL = new TransactionLog(product: new Product(), transactionID: 123L, type: TransactionType.TRADE, price: 102.5, quantity: 100, buySellIndicator: BuySellIndicator.BUY, buyerID: p1, sellerID: p2 ).save()
    def newTL = TransactionLog.get(myTL.id)

    assertEquals newTL.price, 102.5
    assertEquals newTL.quantity, 100
    assertEquals newTL.buySellIndicator, BuySellIndicator.BUY
    assertEquals newTL.buyerID, p1
    assertEquals newTL.sellerID, p2
  }

  void testQuote() {
    mockDomain(TransactionLog)
    def myTL = new TransactionLog(product: new Product(), transactionID: 123L, type: TransactionType.QUOTE, bid: 102.5, bidSize: 200, ask: 105.0, askSize: 400).save()
    println myTL.errors.type
    def newTL = TransactionLog.get(myTL.id)

    assertEquals newTL.bid, 102.5
    assertEquals newTL.bidSize, 200
    assertEquals newTL.ask, 105.0
    assertEquals newTL.askSize, 400

  }
  // Update

  void testUpdateProduct() {
    mockDomain(TransactionLog)
    def myTL = createTL()
    def newTL = TransactionLog.get(myTL.id)
    newTL.setProduct(new Product(name: "ProductB"))

    assertEquals myTL.product.name, "ProductB"
  }

  void testUpdateType() {
    mockDomain(TransactionLog)
    def myTL = createTL()
    def newTL = TransactionLog.get(myTL.id)
    newTL.setType(TransactionType.QUOTE)

    assertEquals myTL.type, TransactionType.QUOTE    
  }

  // Delete

  void testDeletion() {
    mockDomain(TransactionLog)
    def myTL = createTL()
    def delTL = TransactionLog.get(myTL.id)
    delTL.delete(flush:true)

    assertFalse TransactionLog.exists(myTL.id)
  }

}
