/*
 * Copyright 2009-2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an
 *
 * "AS IS" BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package edu.kit.iism.cdamarket.profile

import grails.test.GrailsUnitTestCase

class H0GeneratorTests extends GrailsUnitTestCase {
  H0Generator gen
  Calendar winterSundayMidnight
  Calendar winterWeekdayMidnight
  Calendar intermediateSaturdayMidnight
  Calendar intermediateSundayMidnight
  Calendar summerWeekdayMidnight
  Calendar summerWeekdayZeroFourteenFiftyNine
  Calendar summerWeekdayZeroFifteen
  Calendar summerWeekdayOneTwentyNineFiftyNine
  Calendar summerWeekdayOneThirty
  Calendar summerWeekdayTwentyThreeFiftyNineFiftyNine


  protected void setUp() {
    super.setUp()
    gen = new H0Generator()
    winterSundayMidnight = new GregorianCalendar(2009, Calendar.NOVEMBER, 1, 0, 0)
    winterWeekdayMidnight = new GregorianCalendar(2009, Calendar.MARCH, 20, 0, 0)
    intermediateSaturdayMidnight = new GregorianCalendar(2009, Calendar.MARCH, 21, 0, 0)
    intermediateSundayMidnight = new GregorianCalendar(2009, Calendar.MAY, 31, 0, 0)
    summerWeekdayMidnight = new GregorianCalendar(2009, Calendar.JUNE, 1, 0, 0)
    summerWeekdayZeroFourteenFiftyNine = new GregorianCalendar(2009, Calendar.JUNE, 1, 0, 14, 59)
    summerWeekdayZeroFifteen = new GregorianCalendar(2009, Calendar.JUNE, 1, 0, 15)
    summerWeekdayOneTwentyNineFiftyNine = new GregorianCalendar(2009, Calendar.JUNE, 1, 1, 29, 59)
    summerWeekdayOneThirty = new GregorianCalendar(2009, Calendar.JUNE, 1, 1, 30)
    summerWeekdayTwentyThreeFiftyNineFiftyNine = new GregorianCalendar(2009, Calendar.JUNE, 23, 59, 59)
    summerWeekdayTwentyThreeFiftyNineFiftyNine.set(Calendar.HOUR_OF_DAY, 23)
  }

  void testGetSeasonType() {
    assertEquals (SeasonType.Winter, TimeSeriesUtils.getSeasonType(winterSundayMidnight))
    assertEquals (SeasonType.Winter, TimeSeriesUtils.getSeasonType(winterSundayMidnight.getTime()))
    assertEquals (SeasonType.Winter, TimeSeriesUtils.getSeasonType(winterWeekdayMidnight))
    assertEquals (SeasonType.Winter, TimeSeriesUtils.getSeasonType(winterWeekdayMidnight.getTime()))
    assertEquals (SeasonType.Intermediate, TimeSeriesUtils.getSeasonType(intermediateSaturdayMidnight))
    assertEquals (SeasonType.Intermediate, TimeSeriesUtils.getSeasonType(intermediateSaturdayMidnight.getTime()))
    assertEquals (SeasonType.Intermediate, TimeSeriesUtils.getSeasonType(intermediateSundayMidnight))
    assertEquals (SeasonType.Intermediate, TimeSeriesUtils.getSeasonType(intermediateSundayMidnight.getTime()))
    assertEquals (SeasonType.Summer, TimeSeriesUtils.getSeasonType(summerWeekdayMidnight))
    assertEquals (SeasonType.Summer, TimeSeriesUtils.getSeasonType(summerWeekdayMidnight.getTime()))
  }

  void testGetDayType () {
    assertEquals (DayType.Sunday, TimeSeriesUtils.getDayType(winterSundayMidnight))
    assertEquals (DayType.Sunday, TimeSeriesUtils.getDayType(winterSundayMidnight.getTime()))
    assertEquals (DayType.Sunday, TimeSeriesUtils.getDayType(intermediateSundayMidnight))
    assertEquals (DayType.Sunday, TimeSeriesUtils.getDayType(intermediateSundayMidnight.getTime()))
    assertEquals (DayType.Weekday, TimeSeriesUtils.getDayType(winterWeekdayMidnight))
    assertEquals (DayType.Weekday, TimeSeriesUtils.getDayType(winterWeekdayMidnight.getTime()))
    assertEquals (DayType.Weekday, TimeSeriesUtils.getDayType(summerWeekdayMidnight))
    assertEquals (DayType.Weekday, TimeSeriesUtils.getDayType(summerWeekdayMidnight.getTime()))
    assertEquals (DayType.Saturday, TimeSeriesUtils.getDayType(intermediateSaturdayMidnight))
    assertEquals (DayType.Saturday, TimeSeriesUtils.getDayType(intermediateSaturdayMidnight.getTime()))
  }

  void testGetTimeslot() {
    assertEquals (0, TimeSeriesUtils.getTimeslot(summerWeekdayMidnight))
    assertEquals (0, TimeSeriesUtils.getTimeslot(summerWeekdayZeroFourteenFiftyNine))
    assertEquals (1, TimeSeriesUtils.getTimeslot(summerWeekdayZeroFifteen))
    assertEquals (5, TimeSeriesUtils.getTimeslot(summerWeekdayOneTwentyNineFiftyNine))
    assertEquals (6, TimeSeriesUtils.getTimeslot(summerWeekdayOneThirty))
    assertEquals (95, TimeSeriesUtils.getTimeslot(summerWeekdayTwentyThreeFiftyNineFiftyNine))
  }


  void testGetLoad() {
    assertEquals (73.2d, gen.retrieveLoad(winterSundayMidnight))
    assertEquals (74.9d, gen.retrieveLoad(winterWeekdayMidnight))
    assertEquals (101.5d, gen.retrieveLoad(intermediateSaturdayMidnight))
    assertEquals (80.7d, gen.retrieveLoad(intermediateSundayMidnight))
    assertEquals (96.5d, gen.retrieveLoad(summerWeekdayMidnight))
    assertEquals (96.5d, gen.retrieveLoad(summerWeekdayZeroFourteenFiftyNine))
    assertEquals (86.3d, gen.retrieveLoad(summerWeekdayZeroFifteen))
    assertEquals (58.0d, gen.retrieveLoad(summerWeekdayOneTwentyNineFiftyNine))
    assertEquals (55.3d, gen.retrieveLoad(summerWeekdayOneThirty))
    assertEquals (107.1d, gen.retrieveLoad(summerWeekdayTwentyThreeFiftyNineFiftyNine))
  }
}